import es
import playerlib
import os

oldvals = {}
modellist = {}
materiallist = {}
classlist = {}
indexlist = {}
useridlist = {}
steamidlist = {}
worldmodeluserid = False

def player_disconnect(ev):
	userid = int(ev["userid"])
	if userid in oldvals:
		del oldvals[userid]
	if userid in useridlist:
		del useridlist[userid]

def player_activate(ev):
	userid = int(ev["userid"])
	steamid = ev["es_steamid"]
	if steamid in steamidlist:
		for classname in steamidlist[steamid]:
			if classname != "model":
				addweaponuserid(userid,classname,steamidlist[steamid][classname]["world"],steamidlist[steamid][classname]["view"])
			else:
				addplayeruserid(userid,steamidlist[steamid]["model"])

def player_spawn(ev):
	userid = int(ev["userid"])
	if userid in useridlist:
		if "model" in useridlist[userid]:
			setPlayerModel(userid, useridlist[userid]["model"])

def checkmodelstring(model):
	if model.startswith("models\\") or model.startswith("models/"):
		model = model[7:]
	if not model.endswith(".mdl"):
		model += ".mdl"
	return model

def checkmaterialstring(material):
	if material.startswith("materials\\") or material.startswith("materials/"):
		material = material[10:]
	if not material.endswith("\\"):
		material += "\\"
	return material

def setPlayerModel(userid, model):
	if model == "-1":
		return
	model = checkmodelstring(model)
	index = es.precachemodel("models/%s" % model)
	assert index > 0, ("Unable to precache model, possible invalid model path:  models/%s" % model)
	es.setplayerprop(userid, "CBaseEntity.m_nModelIndex", index)
	es.setplayerprop(userid, "CBaseEntity.m_clrRender", -1)

def setWorldModel(entity, model):	
	if model == "-1":
		return
	model = checkmodelstring(model)
	index = es.precachemodel("models/%s" % model)
	assert index > 0, ("Unable to precache model, possible invalid model path:  models/%s" % model)
	es.setindexprop(entity, "CBaseCombatWeapon.m_iWorldModelIndex", index)
	es.setindexprop(entity, "CBaseEntity.m_clrRender", -1)

def setActiveViewModel(userid, model):
	if model == "-1":
		return
	model = checkmodelstring(model)
	index = es.precachemodel("models/%s" % model)
	assert index > 0, ("Unable to precache model, possible invalid model path:  models/%s" % model)
	viewModelEnt = "predicted_viewmodel"
	gamename = str(es.ServerVar("eventscripts_gamedir")).replace("\\", "/").rpartition("/")[2].lower()
	if gamename == "tf":
		viewModelEnt = "tf_viewmodel"
	viewModelweaponindex = getActiveViewModelweaponindex(userid,viewModelEnt)
	assert viewModelweaponindex > 0, "Unable to get viewmodel index"
	es.setindexprop(viewModelweaponindex, "CBaseViewModel.m_nModelIndex", index)

def getActiveViewModelweaponindex(userid,viewModelEntity):
	playerHandle = es.getplayerhandle(userid)
	entityList = es.createentitylist(viewModelEntity)
	for index in entityList:
		if es.getindexprop(index, "CBaseViewModel.m_hOwner") != playerHandle:
			continue
		if es.getplayerprop(userid, "CBaseCombatCharacter.m_hActiveWeapon") != es.getindexprop(index, "CBaseViewModel.m_hWeapon"):
			continue
		return index
	return -1

def load():
	if es.exists("command","mc_addweaponclass") == 0:
		es.regcmd("mc_addweaponclass", "modelchanger/addweaponclasshandler", "This adds a weapon class")
	if es.exists("command","mc_addweaponindex") == 0:
		es.regcmd("mc_addweaponindex", "modelchanger/addweaponindexhandler", "This adds an weapon index")
	if es.exists("command","mc_addweaponuserid") == 0:
		es.regcmd("mc_addweaponuserid", "modelchanger/addweaponuseridhandler", "This adds a weapon userid")
	if es.exists("command","mc_addweaponsteamid") == 0:
		es.regcmd("mc_addweaponsteamid", "modelchanger/addweaponsteamidhandler", "This adds a weapon steamid")
	if es.exists("command","mc_addplayeruserid") == 0:
		es.regcmd("mc_addplayeruserid", "modelchanger/addplayeruseridhandler", "This adds a player userid")
	if es.exists("command","mc_addplayersteamid") == 0:
		es.regcmd("mc_addplayersteamid", "modelchanger/addplayersteamidhandler", "This adds a player steamid")
	if es.exists("command","mc_delweaponclass") == 0:
		es.regcmd("mc_delweaponclass", "modelchanger/delweaponclasshandler", "This removes a weapon class")
	if es.exists("command","mc_delweaponindex") == 0:
		es.regcmd("mc_delweaponindex", "modelchanger/delweaponindexhandler", "This removes an weapon index")
	if es.exists("command","mc_delweaponuserid") == 0:
		es.regcmd("mc_delweaponuserid", "modelchanger/delweaponuseridhandler", "This removes a weapon userid")
	if es.exists("command","mc_delweaponsteamid") == 0:
		es.regcmd("mc_delweaponsteamid", "modelchanger/delweaponsteamidhandler", "This removes a weapon steamid")
	if es.exists("command","mc_delplayeruserid") == 0:
		es.regcmd("mc_delplayeruserid", "modelchanger/delplayeruseridhandler", "This removes a player userid")
	if es.exists("command","mc_delplayersteamid") == 0:
		es.regcmd("mc_delplayersteamid", "modelchanger/delplayersteamidhandler", "This removes a player steamid")
	if es.exists("command","mc_addmodeldownload") == 0:
		es.regcmd("mc_addmodeldownload", "modelchanger/addmodeldownloadhandler", "This adds a model to be downloaded")
	if es.exists("command","mc_addmaterialdownload") == 0:
		es.regcmd("mc_addmaterialdownload", "modelchanger/addmaterialdownloadhandler", "This adds a material to be downloaded")
	es.addons.registerTickListener(ticker)

def unload():
	es.addons.unregisterTickListener(ticker)

def addweaponclasshandler():
	if es.getargc() > 3:
		classname = es.getargv(1)
		worldmodel = es.getargv(2)
		viewmodel = es.getargv(3)
		addweaponclass(classname,worldmodel,viewmodel)
	else:
		es.dbgmsg(0, "Syntax: mc_addweaponclass <classname> <worldmodel> <viewmodel>")

def addweaponindexhandler():
	if es.getargc() > 3:
		index = int(es.getargv(1))
		worldmodel = es.getargv(2)
		viewmodel = es.getargv(3)
		addweaponindex(index,worldmodel,viewmodel)
	else:
		es.dbgmsg(0, "Syntax: mc_addweaponindex <index> <worldmodel> <viewmodel>")

def addweaponuseridhandler():
	if es.getargc() > 4:
		userid = int(es.getargv(1))
		classname = es.getargv(2)
		worldmodel = es.getargv(3)
		viewmodel = es.getargv(4)
		addweaponuserid(userid,classname,worldmodel,viewmodel)
	else:
		es.dbgmsg(0, "Syntax: mc_addweaponuserid <userid> <class> <worldmodel> <viewmodel>")

def addweaponsteamidhandler():
	if es.getargc() > 4:
		steamid = es.getargv(1)
		classname = es.getargv(2)
		worldmodel = es.getargv(3)
		viewmodel = es.getargv(4)
		addweaponsteamid(steamid,classname,worldmodel,viewmodel)
	else:
		es.dbgmsg(0, "Syntax: mc_addweaponsteamid <steamid> <class> <worldmodel> <viewmodel>")

def addplayeruseridhandler():
	if es.getargc() > 2:
		userid = int(es.getargv(1))
		model = es.getargv(2)
		addplayeruserid(userid,model)
	else:
		es.dbgmsg(0, "Syntax: mc_addplayeruserid <userid> <model>")

def addplayersteamidhandler():
	if es.getargc() > 2:
		steamid = es.getargv(1)
		model = es.getargv(2)
		addplayersteamid(steamid,model)
	else:
		es.dbgmsg(0, "Syntax: mc_addplayersteamid <steamid> <model>")

def delweaponclasshandler():
	if es.getargc() > 1:
		classname = es.getargv(1)
		delweaponclass(classname)
	else:
		es.dbgmsg(0, "Syntax: mc_delweaponclass <classname>")

def delweaponindexhandler():
	if es.getargc() > 1:
		index = int(es.getargv(1))
		delweaponindex(index)
	else:
		es.dbgmsg(0, "Syntax: mc_delweaponindex <index>")

def delweaponuseridhandler():
	if es.getargc() > 2:
		userid = int(es.getargv(1))
		classname = es.getargv(2)
		delweaponuserid(userid,classname)
	else:
		es.dbgmsg(0, "Syntax: mc_delweaponuserid <userid> <classname>")

def delweaponsteamidhandler():
	if es.getargc() > 1:
		steamid = es.getargv(1)
		classname = es.getargv(2)
		delweaponsteamid(steamid,classname)
	else:
		es.dbgmsg(0, "Syntax: mc_delweaponsteamid <steamid> <classname>")

def delplayeruseridhandler():
	if es.getargc() > 1:
		userid = int(es.getargv(1))
		delplayeruserid(userid)
	else:
		es.dbgmsg(0, "Syntax: mc_delplayeruserid <userid>")

def delplayersteamidhandler():
	if es.getargc() > 1:
		steamid = es.getargv(1)
		delplayersteamid(steamid)
	else:
		es.dbgmsg(0, "Syntax: mc_delplayersteamid <steamid>")

def addmodeldownloadhandler():
	if es.getargc() > 1:
		model = es.getargv(1)
		addmodeldownload(model)
	else:
		es.dbgmsg(0, "Syntax: mc_addmodeldownload <model>")

def addmaterialdownloadhandler():
	if es.getargc() > 1:
		material = es.getargv(1)
		addmaterialdownload(material)
	else:
		es.dbgmsg(0, "Syntax: mc_addmaterialdownload <material>")

def addweaponclass(classname,worldmodel,viewmodel):
	if not classname in classlist:
		classlist[classname] = {}
		classlist[classname]["world"] = worldmodel
		classlist[classname]["view"] = viewmodel
		entlist = es.createentitylist(classname)
		for index in entlist:
			setWorldModel(index,worldmodel)
		userlist = es.getUseridList()
		for userid in userlist:
			if not isdead(userid):
				tclassname = getactiveweaponname(userid)
				if tclassname == classname:
					setActiveViewModel(userid,viewmodel)
	else:
		es.dbgmsg(0, "addweaponclass: Specified class already exists. Cannot add.")

def addweaponindex(index,worldmodel,viewmodel):
	if not index in indexlist:
		indexlist[index] = {}
		indexlist[index]["world"] = worldmodel
		indexlist[index]["view"] = viewmodel
		setWorldModel(index,worldmodel)
		userlist = es.getUseridList()
		for userid in userlist:
			if not isdead(userid):
				classname = getactiveweaponname(userid)
				newwepindex = getweaponindex(userid,classname)
				if newwepindex == index:
					setActiveViewModel(userid,viewmodel)
	else:
		es.dbgmsg(0, "addweaponindex: Specified index already exists. Cannot add.")

def addweaponuserid(userid,classname,worldmodel,viewmodel):
	if not userid in useridlist:
		useridlist[userid] = {}
	if not classname in useridlist[userid]:
		useridlist[userid][classname] = {}
		useridlist[userid][classname]["world"] = worldmodel
		useridlist[userid][classname]["view"] = viewmodel
		if not isdead(userid):
			tclassname = getactiveweaponname(userid)
			if tclassname == classname:
				setActiveViewModel(userid,viewmodel)
				index = getweaponindex(userid,classname)
				setWorldModel(index,worldmodel)
	else:
		es.dbgmsg(0, "addweaponuserid: Specified userid[classname] already exists. Cannot add.")

def addweaponsteamid(steamid,classname,worldmodel,viewmodel):
	if not steamid in steamidlist:
		steamidlist[steamid] = {}
	if not classname in steamidlist[steamid]:
		steamidlist[steamid][classname] = {}
		steamidlist[steamid][classname]["world"] = worldmodel
		steamidlist[steamid][classname]["view"] = viewmodel
		userid = int(es.getuserid(steamid))
		if userid > 0:
			addweaponuserid(userid,classname,worldmodel,viewmodel)
	else:
		es.dbgmsg(0, "addweaponsteamid: Specified steamid[classname] already exists. Cannot add.")

def addplayeruserid(userid,model):
	if not userid in useridlist:
		useridlist[userid] = {}
	if not "model" in useridlist[userid]:
		useridlist[userid]["model"] = model
		if not isdead(userid):
			setPlayerModel(userid, model)
	else:
		es.dbgmsg(0, "addplayeruserid: Specified userid['model'] already exists. Cannot add.")

def addplayersteamid(steamid,model):
	if not steamid in steamidlist:
		steamidlist[steamid] = {}
	if not "model" in steamidlist[steamid]:
		steamidlist[steamid]["model"] = model
		userid = int(es.getuserid(steamid))
		if userid > 0:
			addplayeruserid(userid,model)
	else:
		es.dbgmsg(0, "addplayersteamid: Specified steamid['model'] already exists. Cannot add.")

def delweaponclass(classname):
	if classname in classlist:
		del classlist[classname]
	else:
		es.dbgmsg(0, "delweaponclass: Specified class does not exist. Cannot delete.")

def delweaponindex(index):
	if index in indexlist:
		del indexlist[index]
	else:
		es.dbgmsg(0, "delweaponindex: Specified index does not exist. Cannot delete.")

def delweaponuserid(userid,classname):
	if userid in useridlist:
		if classname in useridlist[userid]:
			del useridlist[userid][classname]
		else:
			es.dbgmsg(0, "delweaponuserid: Specified userid[classname] does not exist. Cannot delete.")
	else:
		es.dbgmsg(0, "delweaponuserid: Specified userid[classname] does not exist. Cannot delete.")

def delweaponsteamid(steamid,classname):
	if steamid in steamidlist:
		if classname in steamidlist[steamid]:
			del steamidlist[steamid][classname]
			userid = int(es.getuserid(steamid))
			delweaponuserid(userid,classname)
		else:
			es.dbgmsg(0, "delweaponsteamid: Specified steamid[classname] does not exist. Cannot delete.")
	else:
		es.dbgmsg(0, "delweaponsteamid: Specified steamid[classname] does not exist. Cannot delete.")

def delplayeruserid(userid):
	if userid in useridlist:
		if "model" in useridlist[userid]:
			del useridlist[userid]["model"]
		else:
			es.dbgmsg(0, "delplayeruserid: Specified userid['model'] does not exist. Cannot delete.")
	else:
		es.dbgmsg(0, "delplayeruserid: Specified userid['model'] does not exist. Cannot delete.")

def delplayersteamid(steamid):
	if steamid in steamidlist:
		if "model" in steamidlist[steamid]:
			del steamidlist[steamid]["model"]
			userid = int(es.getuserid(steamid))
			delplayeruserid(userid)
		else:
			es.dbgmsg(0, "delplayersteamid: Specified steamid['model'] does not exist. Cannot delete.")
	else:
		es.dbgmsg(0, "delplayersteamid: Specified steamid['model'] does not exist. Cannot delete.")

def addmodeldownload(model):
	if model not in modellist:
		modellist[model] = True
	model = checkmodelstring(model)
	fileName = es.getAddonPath("modelchanger")
	fileName = fileName.replace("addons\eventscripts/modelchanger", "models\%s"%(model))
	if os.path.exists(fileName):
		es.server.queuecmd("downloadable models\%s"%(model))
		if os.path.exists(fileName.replace(".mdl",".vvd")):
			es.server.queuecmd("downloadable models\%s"%(model.replace(".mdl",".vvd")))
		if os.path.exists(fileName.replace(".mdl",".dx80.vtx")):
			es.server.queuecmd("downloadable models\%s"%(model.replace(".mdl",".dx80.vtx")))
		if os.path.exists(fileName.replace(".mdl",".dx90.vtx")):
			es.server.queuecmd("downloadable models\%s"%(model.replace(".mdl",".dx90.vtx")))
		if os.path.exists(fileName.replace(".mdl",".sw.vtx")):
			es.server.queuecmd("downloadable models\%s"%(model.replace(".mdl",".sw.vtx")))
		if os.path.exists(fileName.replace(".mdl",".xbox.vtx")):
			es.server.queuecmd("downloadable models\%s"%(model.replace(".mdl",".xbox.vtx")))
		if os.path.exists(fileName.replace(".mdl",".phy")):
			es.server.queuecmd("downloadable models\%s"%(model.replace(".mdl",".phy")))		
	else:
		es.dbgmsg(0, "addmodeldownload: Specific model doesn't exist.")

def addmaterialdownload(material):
	if material not in materiallist:
		materiallist[material] = True
	material = checkmaterialstring(material)
	fileName = es.getAddonPath("modelchanger")
	fileName = fileName.replace("addons\eventscripts/modelchanger", "materials\%s"%(material))
	if os.path.exists(fileName):
		for fname in os.listdir(fileName):
			es.server.queuecmd("downloadable materials\%s%s"%(material,fname))
	else:
		es.dbgmsg(0, "addmaterialdownload: Specific material directory doesn't exist.")

def ticker():
	useridlist = es.getUseridList()
	for userid in useridlist:
		checkweapon(userid)

def checkweapon(userid):
	global oldvals
	if userid not in oldvals:
		oldvals[userid] = {}
		oldvals[userid]["windex"] = -1
		oldvals[userid]["whandle"] = -1
		oldvals[userid]["wname"] = ""
	oldweapon = oldvals[userid]["whandle"]
	newweapon = es.getplayerprop(userid, "CBaseCombatCharacter.m_hActiveWeapon")
	if newweapon != oldweapon:
		if not isdead(userid):
			classname = getactiveweaponname(userid)
			newwepindex = getweaponindex(userid,classname)
			oldindex = oldvals[userid]["windex"]
			oldclassname = oldvals[userid]["wname"]
			weapon_changed(userid,oldindex,newwepindex,oldclassname,classname)
			oldvals[userid]["whandle"] = newweapon
			oldvals[userid]["windex"] = newwepindex
			oldvals[userid]["wname"] = classname

def weapon_changed(userid,oldindex,newindex,oldclass,newclass):
	if newclass in classlist:
		setActiveViewModel(userid, classlist[newclass]["view"])
		setWorldModel(newindex, classlist[newclass]["world"])
	elif newindex in indexlist:
		setActiveViewModel(userid, indexlist[newindex]["view"])
		setWorldModel(newindex, indexlist[newindex]["world"])
	elif userid in useridlist:
		if newclass in useridlist[userid]:
			setActiveViewModel(userid, useridlist[userid][newclass]["view"])
			setWorldModel(newindex, useridlist[userid][newclass]["world"])
		if worldmodeluserid and oldclass in useridlist[userid]:
			setWorldModel(oldindex, useridlist[userid][oldclass]["world"])
	if oldclass in classlist:
		setWorldModel(oldindex, classlist[oldclass]["world"])
	elif oldindex in indexlist:
		setWorldModel(oldindex, indexlist[oldindex]["world"])

def es_map_changed(ev):
	download()		

def getactiveweaponname(userid):
	plist = es.createplayerlist(userid)
	return plist[userid]["weapon"]

def getweaponindex(userid,weapon):
	myPlayer = playerlib.getPlayer(userid)
	return myPlayer.get("weaponindex",weapon)

def isdead(userid):
	plist = es.createplayerlist(userid)
	if (plist[userid]["isdead"] == 0):
		return False
	return True

def download():
	for model in modellist:
		addmodeldownload(model)
	for material in materiallist:
		addmaterialdownload(material)